
from BaseDataMaintenance.common.milvusUtil import *
from multiprocessing import Process,Queue
from BaseDataMaintenance.common.Utils import *
from BaseDataMaintenance.common.multiThread import MultiThreadHandler
from apscheduler.schedulers.blocking import BlockingScheduler

from BaseDataMaintenance.model.ots.document_product_dict import *
from BaseDataMaintenance.model.ots.document_product_dict_interface import *
from BaseDataMaintenance.model.ots.document_product import *
from BaseDataMaintenance.model.ots.document_product_tmp import *

from BaseDataMaintenance.dataSource.source import getConnect_ots

from tablestore import *
from BaseDataMaintenance.common.Utils import getRow_ots

from BaseDataMaintenance.maintenance.product.productUtils import *

import time
import traceback
import json
import requests
from random import randint


IS_SYNCHONIZED = 2

class Product_Dict_Manager():

    def __init__(self):

        self.collection_name_name = COLLECTION_NAME_NAME
        self.collection_name_brand = COLLECTION_NAME_BRAND
        self.collection_name_specs = COLLECTION_NAME_SPECS
        self.search_params = {"metric_type":"IP",
                              "params":{"nprobe":10}}

        self.init_milvus()
        self.ots_client = getConnect_ots()
        self.queue_product_dict = Queue()

        self.queue_product_interface = Queue()

        self.session = requests.Session()


        self.Coll_name = getCollection(self.collection_name_name)
        self.Coll_brand = getCollection(self.collection_name_brand)
        self.Coll_specs = getCollection(self.collection_name_specs)

        # self.pool_name = ConnectorPool(init_num=10,max_num=30,method_init=getCollection,collection_name=self.collection_name_name)
        #
        # self.pool_brand = ConnectorPool(init_num=10,max_num=30,method_init=getCollection,collection_name=self.collection_name_brand)
        # self.pool_specs = ConnectorPool(init_num=10,max_num=30,method_init=getCollection,collection_name=self.collection_name_specs)


    def init_milvus(self):
        from pymilvus import connections,FieldSchema,DataType
        fields = [
            # FieldSchema(name="pk_id",dtype=DataType.INT64,is_primary=True,auto_id=True), # pk is the same as ots
            FieldSchema(name="ots_id",dtype=DataType.VARCHAR,max_length=32,is_primary=True),
            FieldSchema(name="ots_name",dtype=DataType.VARCHAR,max_length=MAX_NAME_LENGTH),
            FieldSchema(name="standard_name",dtype=DataType.VARCHAR,max_length=MAX_NAME_LENGTH),
            FieldSchema(name="standard_name_id",dtype=DataType.VARCHAR,max_length=32),
            FieldSchema(name="embedding",dtype=DataType.FLOAT_VECTOR,dim=1024),
            FieldSchema(name="ots_parent_id",dtype=DataType.VARCHAR,max_length=32),
            FieldSchema(name="ots_grade",dtype=DataType.INT64)
        ]

        index_name = "embedding"
        index_params = {"params":{"nlist":2048},
                        "index_type":"IVF_SQ8",
                        "metric_type":"IP"}

        init_milvus(milvus_host)

        #build the product name brand specs embedding respectively
        create_embedding_schema(self.collection_name_name,fields,index_name,index_params)
        create_embedding_schema(self.collection_name_brand,fields,index_name,index_params)
        create_embedding_schema(self.collection_name_specs,fields,index_name,index_params)




    def get_collection(self,grade):
        Coll = None
        Coll_name = None
        if grade ==SPECS_GRADE:
            Coll = self.Coll_specs
            Coll_name = self.collection_name_specs
        if grade == BRAND_GRADE:
            Coll = self.Coll_brand
            Coll_name = self.collection_name_brand
        if grade == NAME_GRADE:
            Coll = self.Coll_name
            Coll_name = self.collection_name_name
        return Coll,Coll_name



    def embedding_producer(self,columns=[DOCUMENT_PRODUCT_DICT_NAME,DOCUMENT_PRODUCT_DICT_PARENT_ID,DOCUMENT_PRODUCT_DICT_GRADE,DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS]):

        bool_query = BoolQuery(
            must_queries=[RangeQuery(DOCUMENT_PRODUCT_DICT_GRADE,3,5,True,True)],
            must_not_queries=[TermQuery(DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED,IS_SYNCHONIZED)])

        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                            SearchQuery(bool_query,sort=Sort(sorters=[FieldSort(DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED)]),limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))

        list_dict = getRow_ots(rows)
        for _d in list_dict:
            self.queue_product_dict.put(_d)

        while next_token:
            rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                                SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))
            list_dict = getRow_ots(rows)
            for _d in list_dict:
                self.queue_product_dict.put(_d)
            if self.queue_product_dict.qsize()>=1000:
                break
        log("product_dict embedding total_count:%d"%total_count)



    def embedding_comsumer(self):
        def handle(item,result_queue):
            try:
                id = item.get(DOCUMENT_PRODUCT_DICT_ID)
                name = str(item.get(DOCUMENT_PRODUCT_DICT_NAME))[:MAX_NAME_LENGTH]
                vector = request_embedding(name)
                parent_id = item.get(DOCUMENT_PRODUCT_DICT_PARENT_ID)
                grade = item.get(DOCUMENT_PRODUCT_DICT_GRADE)
                Coll,_ = self.get_collection(grade)
                standard_alias = item.get(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS,"")
                if vector is not None and Coll is not None:

                    data = [[id],
                            [name],
                            [name],
                            [id],
                            [vector],
                            [parent_id],
                            [grade]]
                    insert_embedding(Coll,data)


                    if standard_alias is not None and standard_alias!="":
                        list_alias = standard_alias.split(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS_SEPARATOR)
                        for _alias in list_alias:
                            _alias = _alias.strip()
                            if len(_alias)==0:
                                continue
                            if _alias==name:
                                continue
                            _id = get_document_product_dict_standard_alias_id(_alias)
                            vector = request_embedding(_alias)
                            data = [[_id],
                                    [_alias],
                                    [name],
                                    [id],
                                    [vector],
                                    [parent_id],
                                    [grade]]
                            insert_embedding(Coll,data)

                    _pd = Document_product_dict({DOCUMENT_PRODUCT_DICT_ID:id,DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED:IS_SYNCHONIZED})
                    _pd.update_row(self.ots_client)


            except Exception as e:
                traceback.print_exc()

        self.embedding_producer()
        start_time = time.time()
        q_size = self.queue_product_dict.qsize()
        mt = MultiThreadHandler(self.queue_product_dict,handle,None,5,1)
        mt.run()
        log("process embedding %d records cost %.2f s"%(q_size,time.time()-start_time))

    def process_history_name(self,list_name,action):
        if action=="insert":
            # search document_product_temp and update status
            # query in blur mode
            for name in list_name:
                should_q = self.make_query(name,DOCUMENT_PRODUCT_TMP_NAME,MatchPhraseQuery,4,5)
                if should_q is None:
                    continue
                bool_query =BoolQuery(must_queries=[
                    RangeQuery(DOCUMENT_PRODUCT_TMP_STATUS,401,450,True,True),
                    should_q
                ])

                rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_temp","document_product_temp_index",
                                                                                    SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("status")]),limit=100,get_total_count=True),
                                                                                    columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                list_data = getRow_ots(rows)
                while next_token:
                    rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_temp","document_product_temp_index",
                                                                                        SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                        columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                    list_data.extend(getRow_ots(rows))
                for _data in list_data:
                    id = _data.get(DOCUMENT_PRODUCT_TMP_ID)
                    status = randint(1,50)
                    _d = {DOCUMENT_PRODUCT_TMP_ID:id,
                          DOCUMENT_PRODUCT_TMP_STATUS:status}
                    _dt = Document_product_tmp(_d)
                    _dt.update_row(self.ots_client)

        elif action=="delete":
            # delete document_product
            # update temp new_id and status to 401-450

            for name in list_name:
                bool_query = self.make_query(name,DOCUMENT_PRODUCT_NAME,TermQuery,len(name),5)
                if bool_query is not None:
                    rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                        SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("status")]),limit=100,get_total_count=True),
                                                                                        columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                    list_data = getRow_ots(rows)
                    while next_token:
                        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                            SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                            columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                        list_data.extend(getRow_ots(rows))
                    for _d in list_data:
                        _id = _d.get(DOCUMENT_PRODUCT_ID)

                        _d = {DOCUMENT_PRODUCT_ID:_id}
                        dp = Document_product(_d)
                        dp.delete_row(self.ots_client)

                        original_id = _d.get(DOCUMENT_PRODUCT_ORIGINAL_ID)
                        _d = {DOCUMENT_PRODUCT_TMP_ID:original_id,DOCUMENT_PRODUCT_TMP_STATUS:randint(401,450)}
                        dpt = Document_product_tmp(_d)
                        dpt.update_row(self.ots_client)


        elif action=="update":
            # delete document_product and update document_product_temp to rerun
            for name in list_name:
                bool_query = self.make_query(name,DOCUMENT_PRODUCT_NAME,TermQuery,len(name),5)
                if bool_query is not None:
                    rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                        SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("status")]),limit=100,get_total_count=True),
                                                                                        columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                    list_data = getRow_ots(rows)
                    while next_token:
                        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                            SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                            columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                        list_data.extend(getRow_ots(rows))
                    for _d in list_data:
                        _id = _d.get(DOCUMENT_PRODUCT_ID)
                        original_id = _d.get(DOCUMENT_PRODUCT_ORIGINAL_ID)
                        self.rerun(_id,original_id)

    def process_history_brand(self,list_brand,action):
        # search document_product and rerun
        for name in list_brand:
            if action=="insert":
                name = re.sub("有限|责任|公司",'',name)
                bool_query = self.make_query(name,DOCUMENT_PRODUCT_ORIGINAL_BRAND,MatchPhraseQuery,4,5)
            else:
                bool_query = self.make_query(name,DOCUMENT_PRODUCT_BRAND,TermQuery,len(name),5)
            if bool_query is not None:
                rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                    SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("status")]),limit=100,get_total_count=True),
                                                                                    columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                list_data = getRow_ots(rows)
                while next_token:
                    rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                        SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                        columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                    list_data.extend(getRow_ots(rows))
                for _d in list_data:
                    _id = _d.get(DOCUMENT_PRODUCT_ID)
                    original_id = _d.get(DOCUMENT_PRODUCT_ORIGINAL_ID)
                    self.rerun(_id,original_id)


    def process_history_specs(self,list_specs,action):
        # search document_product and rerun

        for name in list_specs:
            if action=="insert":
                bool_query = self.make_query(name,DOCUMENT_PRODUCT_ORIGINAL_SPECS,MatchPhraseQuery,len(name),5)
            else:
                bool_query = self.make_query(name,DOCUMENT_PRODUCT_SPECS,TermQuery,len(name),5)
            if bool_query is not None:
                rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                    SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("status")]),limit=100,get_total_count=True),
                                                                                    columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                list_data = getRow_ots(rows)
                while next_token:
                    rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                        SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                        columns_to_get=ColumnsToGet([DOCUMENT_PRODUCT_ORIGINAL_ID],return_type=ColumnReturnType.SPECIFIED))
                    list_data.extend(getRow_ots(rows))
                for _d in list_data:
                    _id = _d.get(DOCUMENT_PRODUCT_ID)
                    original_id = _d.get(DOCUMENT_PRODUCT_ORIGINAL_ID)
                    self.rerun(_id,original_id)

    def rerun(self,id,original_id):
        _d = {DOCUMENT_PRODUCT_TMP_ID:original_id,DOCUMENT_PRODUCT_TMP_STATUS:1}
        dpt = Document_product_tmp(_d)
        dpt.update_row(self.ots_client)

        _d = {DOCUMENT_PRODUCT_ID:id}
        dp = Document_product(_d)
        dp.delete_row(self.ots_client)


    def make_query(self,name,column,query_type,min_len,strides):

        should_q = []
        strides_spce = len(name)-min_len+1
        for _i in range(min(strides_spce,strides)):
            _s = str(name)[_i:min_len]
            if query_type==WildcardQuery:
                should_q.append(query_type(column,"*%s*"%_s))
            elif query_type==TermQuery or query_type==MatchPhraseQuery:
                should_q.append(query_type(column,"%s"%(_s)))
        if len(should_q)>0:
            return BoolQuery(should_queries=should_q)
        return None




    def process_history(self,list_name,grade,action):
        if grade==NAME_GRADE:
            self.process_history_name(list_name,action)
        elif grade==BRAND_GRADE:
            self.process_history_brand(list_name,action)
        elif grade==SPECS_GRADE:
            self.process_history_specs(list_name,action)

    def exists_records(self,name,grade,create_time):
        term_columns = None
        if grade==NAME_GRADE:
            term_columns = DOCUMENT_PRODUCT_NAME
        elif grade==BRAND_GRADE:
            term_columns = DOCUMENT_PRODUCT_BRAND
        elif grade==SPECS_GRADE:
            term_columns = DOCUMENT_PRODUCT_SPECS
        if term_columns is not None:
            bool_query = BoolQuery(must_queries=[
                TermQuery(term_columns,str(name)),
                RangeQuery(DOCUMENT_PRODUCT_DICT_CREATE_TIME,None,str(create_time))
            ])
            rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product","document_product_index",
                                                                                SearchQuery(bool_query,get_total_count=True,limit=1),
                                                                                columns_to_get=ColumnsToGet(return_type=ColumnReturnType.NONE))
            if total_count>0:
                return True
        return False


    def act_insert(self,name,alias,grade,original_id,parent_id,standard_alias,create_time):


        #update document_product_dict
        if original_id is None or original_id=="":
            if parent_id is None:
                original_id = get_document_product_dict_id("",name)
            else:
                original_id = get_document_product_dict_id(parent_id,name)
        if parent_id is not None and parent_id!="":
            _d = {DOCUMENT_PRODUCT_DICT_ID:original_id,
                  DOCUMENT_PRODUCT_DICT_ALIAS:alias,
                  DOCUMENT_PRODUCT_DICT_NAME:name,
                  DOCUMENT_PRODUCT_DICT_GRADE:grade,
                  DOCUMENT_PRODUCT_DICT_PARENT_ID:parent_id,
                  DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS:standard_alias,
                  DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED:IS_SYNCHONIZED,
                  DOCUMENT_PRODUCT_DICT_CREATE_TIME:getCurrent_date(format="%Y-%m-%d %H:%M:%S"),
                  DOCUMENT_PRODUCT_DICT_UPDATE_TIME:getCurrent_date(format="%Y-%m-%d %H:%M:%S")}
            _dpd = Document_product_dict(_d)
            _dpd.update_row(self.ots_client)

        # search interface if name and grade exists then update document_product_dict and return
        bool_query = BoolQuery(must_queries=[
            TermQuery(DOCUMENT_PRODUCT_DICT_INTERFACE_NAME,name),
            TermQuery(DOCUMENT_PRODUCT_DICT_INTERFACE_GRADE,grade),
            RangeQuery(DOCUMENT_PRODUCT_DICT_INTERFACE_STATUS,201,301)
        ])
        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict_interface","document_product_dict_interface_index",
                                                                            SearchQuery(bool_query,get_total_count=True))
        if total_count>0:
            return

        list_name = []
        #update milvus
        vector = request_embedding(name)
        Coll,_ = self.get_collection(grade)
        if vector is not None and Coll is not None:
            id = original_id
            data = [[id],
                    [name],
                    [name],
                    [id],
                    [vector],
                    [parent_id],
                    [grade]]
            insert_embedding(Coll,data)
            list_name.append(name)

            if standard_alias is not None and standard_alias!="":
                list_alias = standard_alias.split(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS_SEPARATOR)
                for _alias in list_alias:
                    _alias = _alias.strip()
                    if len(_alias)==0:
                        continue
                    if _alias==name:
                        continue
                    _id = get_document_product_dict_standard_alias_id(_alias)
                    vector = request_embedding(_alias)
                    data = [[_id],
                            [_alias],
                            [name],
                            [id],
                            [vector],
                            [parent_id],
                            [grade]]
                    insert_embedding(Coll,data)
                    list_name.append(_alias)

        #judge whether there exists records before this record created,if not process the history data
        if not self.exists_records(name,grade,create_time):
            self.process_history(list_name,grade,"insert")


    def act_update(self,name,alias,grade,original_id,parent_id,standard_alias,create_time):
        # check whether there are change variable
        if original_id is None or original_id=="":
            return
        _d = {DOCUMENT_PRODUCT_DICT_ID:original_id}
        dpd = Document_product_dict(_d)
        if not dpd.fix_columns(self.ots_client,[DOCUMENT_PRODUCT_DICT_NAME,DOCUMENT_PRODUCT_DICT_ALIAS,DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS,DOCUMENT_PRODUCT_DICT_PARENT_ID,DOCUMENT_PRODUCT_DICT_CREATE_TIME,DOCUMENT_PRODUCT_DICT_UPDATE_TIME,DOCUMENT_PRODUCT_DICT_STATUS],True):
            return

        if parent_id is None or parent_id=="":
            parent_id = dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_PARENT_ID)
        old_name = dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_NAME)
        old_name_set = set([old_name])
        _alias = dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS)
        if _alias is not None:
            for s in _alias.split(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS_SEPARATOR):
                if s!="":
                    old_name_set.add(s)

        new_name_set = set([name])
        if standard_alias is not None:
            for s in standard_alias.split(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS_SEPARATOR):
                if s!="":
                    new_name_set.add(s)

        if old_name!=name:
            new_id = get_document_product_dict_id(parent_id, name)
        else:
            new_id = original_id

        # update the milvus
        if not (len(new_name_set)==len(old_name_set) and len(new_name_set)==len(new_name_set&old_name_set)):
            Coll,_ = self.get_collection(grade)
            o_id = original_id
            expr = " ots_id in ['%s']"%o_id
            Coll.delete(expr)

            _alias = dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS)
            if _alias is not None and _alias!="":
                list_alias = _alias.split(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS_SEPARATOR)
                for _alias in list_alias:
                    _alias = _alias.strip()
                    if len(_alias)==0:
                        continue
                    if _alias==name:
                        continue
                    _id = get_document_product_dict_standard_alias_id(_alias)
                    expr = " ots_id in ['%s']"%o_id
                    Coll.delete(expr)

            list_name = []
            vector = request_embedding(name)
            if vector is not None and Coll is not None:
                id = new_id
                data = [[id],
                        [name],
                        [name],
                        [id],
                        [vector],
                        [parent_id],
                        [grade]]
                insert_embedding(Coll,data)
                list_name.append(name)

                if standard_alias is not None and standard_alias!="":
                    list_alias = standard_alias.split(DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS_SEPARATOR)
                    for _alias in list_alias:
                        _alias = _alias.strip()
                        if len(_alias)==0:
                            continue
                        if _alias==name:
                            continue
                        _id = get_document_product_dict_standard_alias_id(_alias)
                        vector = request_embedding(_alias)
                        data = [[_id],
                                [_alias],
                                [name],
                                [id],
                                [vector],
                                [parent_id],
                                [grade]]
                        insert_embedding(Coll,data)
                        list_name.append(_alias)

        # process history
        delete_names = list(old_name_set-new_name_set)
        if len(delete_names)>0:
            self.process_history([old_name],grade,"update")
        insert_names = list(new_name_set-old_name_set)
        if len(insert_names)>0:
            self.process_history(insert_names,grade,"insert")


        # update document_product_dict
        _d = {DOCUMENT_PRODUCT_DICT_ID:new_id,
              DOCUMENT_PRODUCT_DICT_NAME:name,
              DOCUMENT_PRODUCT_DICT_GRADE:grade,
              DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED:IS_SYNCHONIZED,
              DOCUMENT_PRODUCT_DICT_STANDARD_ALIAS:standard_alias}

        if alias is not None and alias!="":
            _d[DOCUMENT_PRODUCT_DICT_ALIAS] = alias
        if parent_id is not None and parent_id!="":
            _d[DOCUMENT_PRODUCT_DICT_PARENT_ID] = parent_id
        if old_name!=name:
            _d[DOCUMENT_PRODUCT_DICT_CREATE_TIME] = dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_CREATE_TIME)
            _d[DOCUMENT_PRODUCT_DICT_UPDATE_TIME] = getCurrent_date(format="%Y-%m-%d %H:%M:%S")
            _d[DOCUMENT_PRODUCT_DICT_STATUS] = dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_STATUS)



        dpd = Document_product_dict(_d)
        dpd.update_row(self.ots_client)

        if old_name!=name:
            # in the case of name changed ,delete the old name row
            _d = {DOCUMENT_PRODUCT_DICT_ID:original_id}
            dpd = Document_product_dict(_d)

            self.recurse_update_dict(original_id,new_id)

            dpd.delete_row(self.ots_client)
            # change the next level parent_id


    def recurse_update_dict(self,parent_id,new_parent_id):
        bool_query = BoolQuery(must_queries=[
            TermQuery(DOCUMENT_PRODUCT_DICT_PARENT_ID,parent_id)
        ])
        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                            SearchQuery(bool_query,sort=Sort(sorters=[FieldSort(DOCUMENT_PRODUCT_DICT_PARENT_ID)]),limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(return_type=ColumnReturnType.ALL))

        list_data = getRow_ots(rows)
        while next_token:
            rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                                SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                columns_to_get=ColumnsToGet(return_type=ColumnReturnType.ALL))

            list_data.extend(getRow_ots(rows))
        for _data in list_data:
            dpd = Document_product_dict(_data)
            old_id = dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_ID)
            new_id = get_document_product_dict_id(new_parent_id,dpd.getProperties().get(DOCUMENT_PRODUCT_DICT_NAME))

            self.recurse_update_dict(old_id,new_id)

            dpd.setValue(DOCUMENT_PRODUCT_DICT_PARENT_ID,new_parent_id,True)
            dpd.setValue(DOCUMENT_PRODUCT_DICT_ID,new_id,True)
            dpd.update_row(self.ots_client)

            dpd = Document_product_dict({DOCUMENT_PRODUCT_DICT_ID:old_id})
            dpd.delete_row(self.ots_client)

    def act_delete(self,name,alias,grade,original_id,parent_id,standard_alias,create_time):
        #search records which name=name and grade=grade
        bool_query = BoolQuery(must_queries=[
            TermQuery(DOCUMENT_PRODUCT_DICT_NAME,str(name)),
            TermQuery(DOCUMENT_PRODUCT_DICT_GRADE,grade)
        ])

        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                            SearchQuery(bool_query,sort=Sort(sorters=[FieldSort("status")]),limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(return_type=ColumnReturnType.ALL))
        if total_count==0:
            return
        list_data = getRow_ots(rows)
        while next_token:
            rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                                SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                columns_to_get=ColumnsToGet(return_type=ColumnReturnType.ALL))
            list_data.extend(getRow_ots(rows))

        #delete milvus records
        Coll,_ = self.get_collection(grade)
        for _data in list_data:
            o_id = _data.get(DOCUMENT_PRODUCT_DICT_ID)
            expr = " ots_id in ['%s']"%o_id
            Coll.delete(expr)

        #process_history data
        self.process_history([name],grade,"delete")

        #delete document_product_dict
        log("delete document_product_dict name:%s grade:%s count:%s"%(str(name),str(grade),str(len(list_data))))
        for _data in list_data:
            id = _data.get(DOCUMENT_PRODUCT_DICT_ID)

            self.recurse_delete_dict(id)

            _d = {DOCUMENT_PRODUCT_DICT_ID:id}
            dpd = Document_product_dict(_d)
            dpd.delete_row(self.ots_client)



    def recurse_delete_dict(self,id):
        bool_query = BoolQuery(must_queries=[
            TermQuery(DOCUMENT_PRODUCT_DICT_PARENT_ID,id)
        ])
        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                            SearchQuery(bool_query,sort=Sort(sorters=[FieldSort(DOCUMENT_PRODUCT_DICT_PARENT_ID)]),limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(return_type=ColumnReturnType.NONE))

        list_data = getRow_ots(rows)
        while next_token:
            rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict","document_product_dict_index",
                                                                                SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                columns_to_get=ColumnsToGet(return_type=ColumnReturnType.NONE))

            list_data.extend(getRow_ots(rows))
        for _data in list_data:
            _id = _data.get(DOCUMENT_PRODUCT_DICT_ID)
            self.recurse_delete_dict(_id)
            dpd = Document_product_dict(_data)
            dpd.delete_row(self.ots_client)


    def embedding_interface_producer(self):

        bool_query = BoolQuery(must_queries=[
            # TermQuery("name",'济南鑫驰'),
            RangeQuery("status",1,50,True,True)
        ])

        rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict_interface","document_product_dict_interface_index",
                                                                            SearchQuery(bool_query,sort=Sort(sorters=[FieldSort(DOCUMENT_PRODUCT_DICT_INTERFACE_CREATE_TIME)]),limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(return_type=ColumnReturnType.ALL))
        list_data = getRow_ots(rows)
        for _data in list_data:
            self.queue_product_interface.put(_data)

        while next_token:
            rows,next_token,total_count,is_all_succeed = self.ots_client.search("document_product_dict_interface","document_product_dict_interface_index",
                                                                                SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                                columns_to_get=ColumnsToGet(return_type=ColumnReturnType.ALL))
            list_data = getRow_ots(rows)
            for _data in list_data:
                self.queue_product_interface.put(_data)
            if self.queue_product_dict.qsize()>1000:
                break
        log("embedding interface total_count %d"%(total_count))

    def embedding_interface_comsumer(self):
        def _handle(item,result_queue):
            id = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_ID)
            action = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_ACTION)
            name = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_NAME)
            alias = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_ALIAS)
            grade = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_GRADE)
            original_id = item.get(DOCUMENT_PRODUCT_DICT_ORIGINAL_ID)
            parent_id = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_PARENT_ID)
            standard_alias = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_STANDARD_ALIAS)
            create_time = item.get(DOCUMENT_PRODUCT_DICT_INTERFACE_CREATE_TIME)

            if name is not None and len(name)>1 and len(name)<MAX_NAME_LENGTH:
                if action=="insert":
                    self.act_insert(name,alias,grade,original_id,parent_id,standard_alias,create_time)
                elif action=="update":
                    self.act_update(name,alias,grade,original_id,parent_id,standard_alias,create_time)
                elif action=="delete":
                    self.act_delete(name,alias,grade,original_id,parent_id,standard_alias,create_time)

            _pdi = Document_product_dict_interface({DOCUMENT_PRODUCT_DICT_INTERFACE_ID:id,
                                                    DOCUMENT_PRODUCT_DICT_INTERFACE_STATUS:randint(201,300)})
            _pdi.update_row(self.ots_client)

        self.embedding_interface_producer()

        mt = MultiThreadHandler(self.queue_product_interface,_handle,None,20,1)
        mt.run()



    def start_embedding_product_dict(self):
        from apscheduler.schedulers.blocking import BlockingScheduler
        scheduler = BlockingScheduler()
        # scheduler.add_job(func=self.embedding_producer,trigger="cron",minute="*/1")
        scheduler.add_job(func=self.embedding_comsumer,trigger="cron",second="*/5")
        scheduler.start()

    def delete_collections(self):
        drop_embedding_collection(self.collection_name_name)
        drop_embedding_collection(self.collection_name_brand)
        drop_embedding_collection(self.collection_name_specs)

def start_embedding_product_dict():
    pdm = Product_Dict_Manager()
    pdm.start_embedding_product_dict()

def drop_product_dict_collections():
    pdm = Product_Dict_Manager()
    pdm.delete_collections()


def search_similar():

    task_queue = Queue()
    ots_client = getConnect_ots()
    pdm = Product_Dict_Manager()

    list_data = []

    columns=[DOCUMENT_PRODUCT_DICT_NAME,DOCUMENT_PRODUCT_DICT_PARENT_ID,DOCUMENT_PRODUCT_DICT_GRADE]

    bool_query = BoolQuery(
        must_queries=[RangeQuery(DOCUMENT_PRODUCT_DICT_GRADE,5,5,True,True)]
    )

    rows,next_token,total_count,is_all_succeed = ots_client.search("document_product_dict","document_product_dict_index",
                                                                        SearchQuery(bool_query,sort=Sort(sorters=[FieldSort(DOCUMENT_PRODUCT_DICT_IS_SYNCHONIZED)]),limit=100,get_total_count=True),
                                                                        columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))

    list_dict = getRow_ots(rows)
    for _d in list_dict:
        list_data.append(_d)

    while next_token:
        rows,next_token,total_count,is_all_succeed = ots_client.search("document_product_dict","document_product_dict_index",
                                                                            SearchQuery(bool_query,next_token=next_token,limit=100,get_total_count=True),
                                                                            columns_to_get=ColumnsToGet(columns,ColumnReturnType.SPECIFIED))
        list_dict = getRow_ots(rows)
        for _d in list_dict:
            list_data.append(_d)
        if len(list_data)>=100000:
            break
    log("product_dict embedding total_count:%d"%total_count)
    for _d in list_data:
        task_queue.put(_d)

    result_queue = Queue()

    def handle(item,result_queue):
        id = item.get(DOCUMENT_PRODUCT_DICT_ID)
        name = item.get(DOCUMENT_PRODUCT_DICT_NAME)
        vector = pdm.request_embedding(name)
        parent_id = item.get(DOCUMENT_PRODUCT_DICT_PARENT_ID)
        grade = item.get(DOCUMENT_PRODUCT_DICT_GRADE)
        Coll,Coll_name = pdm.get_collection(grade)
        output_fields = ['ots_id','ots_name',"ots_parent_id"]
        if vector is not None and Coll is not None:
            search_list = search_embedding(Coll,embedding_index_name,[vector],pdm.search_params,output_fields,limit=10)
            for _item in search_list:
                ots_id = _item.id
                ots_name = _item.entity.get("ots_name")
                ots_parent_id = _item.entity.get("ots_parent_id")
                if name!=ots_name:
                    if is_similar(name,ots_name):
                        _d = {"source_id":id,"source_name":name,"grade":grade,"target_id":ots_id,"target_name":ots_name,"parent_id":parent_id,"target_parent_id":ots_parent_id}
                        result_queue.put(_d)


    mt = MultiThreadHandler(task_queue,handle,result_queue,5,1)
    mt.run()

    df_data = {}
    _set = set()
    df_columns = ["source_id","source_name","grade","parent_id","target_id","target_name","parent_parent_id"]
    while 1:
        try:
            item = result_queue.get(timeout=1)
            source_name = item.get("source_name")
            target_name = item.get("target_name")
            _key1 = "%s-%s"%(source_name,target_name)
            _key2 = "%s-%s"%(target_name,source_name)
            for c in df_columns:
                if c not in df_data:
                    df_data[c] = []
                df_data[c].append(getLegal_str(item.get(c)))

        except Exception as e:
            break
    import pandas as pd
    df = pd.DataFrame(df_data)
    df.to_excel("search_similar1.xlsx",columns=df_columns)


def insert_interface_delete(name,grade):
    ots_client = getConnect_ots()
    from uuid import uuid4
    _d = {DOCUMENT_PRODUCT_DICT_INTERFACE_NAME:name,
          DOCUMENT_PRODUCT_DICT_INTERFACE_STATUS:1,
          DOCUMENT_PRODUCT_DICT_INTERFACE_GRADE:grade,
          DOCUMENT_PRODUCT_DICT_INTERFACE_ID:uuid4().hex,
          DOCUMENT_PRODUCT_DICT_INTERFACE_ACTION:"delete",
          DOCUMENT_PRODUCT_DICT_INTERFACE_CREATE_TIME:getCurrent_date(format="%Y-%m-%d %H:%M:%S")}
    dpdi = Document_product_dict_interface(_d)
    dpdi.update_row(ots_client)


if __name__ == '__main__':
    # start_embedding_product_dict()
    insert_interface_delete("液晶显示屏",4)